
# `docker_rm(options, callback)`

Remove one or more containers. Containers need to be stopped to be deleted unless
force options is set.

## Options

*   `container` (string)   
    Name/ID of the container. MANDATORY   
*   `machine` (string)   
    Name of the docker-machine. MANDATORY if docker-machine installed   
*   `link` (boolean)   
    Remove the specified link   
*   `volumes` (boolean)   
    Remove the volumes associated with the container   
*   `force` (boolean)   
    Force the removal of a running container (uses SIGKILL)   

## Example

## Source Code

    module.exports = (options, callback) ->
      # Validate parameters and madatory conditions
      return callback  Error 'Missing container parameter' unless options.container?
      docker.get_provider options, (err,  provider) =>
        return callback err if err
        options.provider = provider
        cmd = docker.prepare_cmd provider, options.machine
        return callback cmd if util.isError cmd
        cmd += 'docker rm '
        for opt in ['link', 'volumes', 'force']
          cmd += "-#{opt.charAt 0} " if options[opt]
        cmd += options.container
        # Construct other exec parameter
        opts = docker.get_options cmd, options
        @execute opts, (err, executed, stdout, stderr) -> callback err, executed, stdout, stderr
        
## Modules Dependencies

    docker = require './commons'
    util = require 'util'
